import java.io.IOException;
import java.util.HashMap;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IFileEditorMapping;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.widgets.ColumnLayout;
import org.eclipse.ui.forms.widgets.Form;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;

import books.Author;
import books.Book;
import books.BookDB;
import books.BooksFactory;
import books.Genre;
import books.Novel;
import books.TextBook;


public class BookEditor extends EditorPart {
	
	FormToolkit toolkit;
	Form form;
	TableViewer viewer;
	
	BookDB bookDB;
	Resource resource;
	BooksFactory factory;
	
	boolean dirty;
	
	IStructuredContentProvider contentProvider = new IStructuredContentProvider() {
		
		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {	}
		
		@Override
		public void dispose() {	}
		
		@Override
		public Object[] getElements(Object inputElement) {
			return bookDB.getBooks().toArray();
		}
	};
	
	ColumnLabelProvider labelProvider = new ColumnLabelProvider() {

		@Override
		public String getText(Object element) {
			Book book = (Book)element;
			Author author = book.getAuthor().get(0);
			return String.format("%s %s: %s", author.getFirstName(), author.getLastName(), book.getTitle());
		}
		
	};
	
	private void modelUpdated() {
		dirty = true;
		firePropertyChange(PROP_DIRTY);
		viewer.refresh();
	}

	@Override
	public void doSave(IProgressMonitor monitor) {
		try {
			resource.save(new HashMap<String, Boolean>());
			dirty = false;
			firePropertyChange(PROP_DIRTY);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	@Override
	public void doSaveAs() {}

	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
		FileEditorInput fileInput = (FileEditorInput) input;
		
		ResourceSet set = new ResourceSetImpl();
		resource = set.createResource(URI.createFileURI(fileInput.getPath().toString()));
		try {
			resource.load(null);
			bookDB = (BookDB) (resource.getContents().get(0));
			dirty = false;
			factory = BooksFactory.eINSTANCE;
		} catch (IOException e) {
			throw new PartInitException("Cannot load model", e);
		}
	}

	@Override
	public boolean isDirty() {
		return dirty;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void createPartControl(Composite parent) {
		toolkit = new FormToolkit(parent.getDisplay());
		form = toolkit.createForm(parent);
		toolkit.decorateFormHeading(form);
		form.setText("Book Manager");
		Composite body = form.getBody();
		body.setLayout(new ColumnLayout());
		
		//Book list section
		Section booksSection = toolkit.createSection(body, Section.DESCRIPTION);
		booksSection.setText("Books");
		//booksSection.setLayout(new FillLayout());
		Table table = toolkit.createTable(booksSection, SWT.FULL_SELECTION);
		booksSection.setClient(table);
		viewer = new TableViewer(table);
		TableViewerColumn column = new TableViewerColumn(viewer, SWT.LEFT);
		column.setLabelProvider(labelProvider);
		viewer.setContentProvider(contentProvider);
		viewer.setInput(bookDB);
		TableLayout layout = new TableLayout();
		layout.addColumnData(new ColumnWeightData(100));
		table.setLayout(layout);
		table.setHeaderVisible(false);
		table.setLinesVisible(true);
		booksSection.setSize(300, 300);
		
		Section buttonSection = toolkit.createSection(body, Section.DESCRIPTION);
		buttonSection.setText("Controls");
		buttonSection.setSize(300, 300);
		Composite buttonComposite = toolkit.createComposite(buttonSection);
		buttonSection.setClient(buttonComposite);
		buttonComposite.setLayout(new RowLayout());
		Button addButton = toolkit.createButton(buttonComposite, "Add Novel", SWT.NONE);
		addButton.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				Novel novel = factory.createNovel();
				novel.setTitle("New Novel");
				novel.setGenre(Genre.CLASSIC);
				novel.setPages(250);
				novel.getAuthor().add(bookDB.getAuthors().get(0));
				bookDB.getBooks().add(novel);
				modelUpdated();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
		Button addTextbookButton = toolkit.createButton(buttonComposite, "Add Textbook", SWT.NONE);
		addTextbookButton.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				TextBook textbook = factory.createTextBook();
				textbook.setTitle("New Novel");
				textbook.setPages(250);
				Author author = factory.createAuthor();
				author.setFirstName("Textbook");
				author.setLastName("Writer");
				textbook.getAuthor().add(author);
				bookDB.getBooks().add(textbook);
				bookDB.getAuthors().add(author);
				modelUpdated();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
		
		getSite().setSelectionProvider(viewer);
	}

	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	@Override
	public void dispose() {
		super.dispose();
		toolkit.dispose();
	}

}
